'''
This code implements the MDP in Figure 17.1.

The results don't match unless we add Steven Klebanoff's modifications, which are now integrated into the AIMA code.
See: https://github.com/steveklebanoff/AIMA-Python-Reinforcement-Learning

@author: kvlinden
@version Jan 18, 2013
'''

from aima.mdp import best_policy, value_iteration
from aima.utils import print_table
from klebanoff.passive_adp import GridMDP

# Build the text example from Figure 17.1 (taken from AIMA Python).
# Note that, unlike the text, the cells are addressed using zero-based coordinates.
grid = GridMDP([[-0.04, -0.04, -0.04, +1.00],
                [-0.04, None, -0.04, -1.00],
                [-0.04, -0.04, -0.04, -0.04]],
                terminals=[(3, 2), (3, 1)],
               gamma=1.0)

# Compute the policy using delta==0.0000001 to get values close to those in Figures 17.2/3 & 21.1.
values = value_iteration(grid, 0.0000001)
pi = best_policy(grid, values)

# This matches Figure 17.2.a.
print "Optimal Policy:"
print_table(grid.to_arrows(pi))

# This matches Figure 17.3.
print "\nUtilities:"
print_table(grid.to_grid(values))

